/*###############################################################################
## ETiles project
##
## Copyright (C) 2010 Carlos Villegas Núñez  carlos.villegasnunez@alum.uca.es
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################*/

#ifndef MAPA_H
#define MAPA_H

#include <QList>
#include <QPoint>
#include <QPixmap>
#include <QWidget>
#include <QLineEdit>
#include <QDrag>
#include "ventanaprincipal.h"
#include "minimapa.h"
#include "tile.h"

class QDragEnterEvent;
class QDropEvent;
class QMouseEvent;
class Minimapa;
class Tile;
class VentanaPrincipal;

class Mapa : public QLabel
{
    Q_OBJECT

 public:
     /** Constructor de la clase mapa, inicializa los datos necesarios y la superficie donde se representa.
       * @param parent Puntero a la ventana principal.
       * @param tanchura Anchura del patrón del tile.
       * @param taltura Altura del patrón del tile.
       * @param tnumanchura Número de tiles a lo ancho.
       * @param tnumaltura Número de tiles a lo alto.
       */
     Mapa(VentanaPrincipal *parent ,int tanchura,int taltura, int tnumanchura, int tnumaltura);
     /** Destructor de la clase mapa, elimina los datos dinámicos.*/
     ~Mapa();
     /** Método constante que devuelve la anchura del patrón de tile.*/
     int AnchuraTile() const { return tanchura;}
     /** Procedimiento que modifica la anchura del patrón del tile.
     * @param a Nueva anchura.*/
     void setAnchuraTile(int a) { tanchura=a;}
     /** Método constante que devuelve la altura del patrón de tile*/
     int AlturaTile() const { return taltura;}
     /** Procedimiento que modifica la altura del patrón del tile.
     * @param a Nueva altura.*/
     void setAlturaTile(int a) { taltura=a;}
     /** Método constante que devuelve la anchura del patrón de tile con la aplicación del zoom actual*/
     int AnchuraTileZoom() const { return tanchuraZ;}
     /** Método constante que devuelve la altura del patrón de tile con la aplicación del zoom actual*/
     int AlturaTileZoom() const { return talturaZ;}
     /** Método constante que devuelve el número de tiles a lo ancho*/
     int NumAnchuraTile() const { return tnumanchura;}
     /** Procedimiento que modifica el número de tiles a lo ancho.
     * @param a Nuevo número de tiles a lo ancho.*/
     void setNumAnchuraTile(int n) {tnumanchura=n;}
     /** Método constante que devuelve el número de tiles a lo largo*/
     int NumAlturaTile() const { return tnumaltura;}
     /** Procedimiento que modifica el número de tiles a lo alto.
     * @param a Nuevo número de tiles a lo alto.*/
     void setNumAlturaTile(int n) { tnumaltura=n;}
     /** Método que devuelve el tile correspondiente a una posición.*/
     int TilePosicion(int x, int y) const {return matrizTiles[x][y];}
     /** Procedimiento que establece el tile de una posición*/
     void setTilePosicion(int valor,int x, int y) { matrizTiles[x][y] = valor;}
     /** Método que devulve el rectángulo correspondiente a una posición*/
     QRect RectanguloPosicion(int x, int y) const { return QRect(x*AnchuraTileZoom(),y*AlturaTileZoom(),AnchuraTileZoom(),AlturaTileZoom());}
     /** Método que devuelve la imagen correspondiente a un tile*/
     QPixmap ImagenPosicion(int i) const { return listatiles->ImagenTile(i);}
     /** Procedimiento que establece el color de la rejilla*/
     void setColorRejilla(QColor c) { crejilla = c;}
     /** Procedimiento que establece el color del fondo del mapa*/
     void setColorFondo(QColor f) { cfondo = f;}
     /** Procedimiento que establece el color del cuadro de selección*/
     void setColorSel(QColor s) { csel = s;}
     /** Método que devuelve el color de la rejilla*/
     QColor ColorRejilla() const { return crejilla; }
     /** Método que devuelve el color de fondo del mapa*/
     QColor ColorFondo() const { return cfondo;}
     /** Método que devuelve el color del cuadro de selección*/
     QColor ColorSel() const { return csel; }
     /** Procedimiento que modifica el zoom del mapa según el parámetro.
      * @param factor Número que indica si se produce un aumento o descenso en el zoom*/
     void Zoom(double factor);
     /** Método que devuelve si el mapa fue modificado desde la última vez que se guardo*/
     int ModificadoDesdeGuardado() const { return modificado;}
     /** Procedimiento para indicar que se ha guardado el mapa*/
     void MapaGuardado() { modificado = 0; }
     /** Procedimiento que asigna la lista de tiles al mapa*/
     void AsignarListaTile(Tile *tiles);
     /** Procedimiento que establece el minimapa del mapa*/
     void setMinimapa(Minimapa *m) { minimapa=m;}
     /** Procedimiento que activa o desactiva la herramienta Seleccion*/
     void SeleccionActivar(bool);
     /**  Método que devuelve si la herramienta Seleccion está activa*/
     bool SeleccionActiva() const {return seleccion;}
     /** Procedimiento que activa o desactiva la herramienta Relleno*/
     void RellenoActivar(bool);
     /**  Método que devuelve si la herramienta Rellenar está activa*/
     bool RellenoActivo() const {return relleno;}
     /** Procedimiento que activa o desactiva la herramienta Borrar*/
     void BorrarActivar(bool);
     /**  Método que devuelve si la herramienta Borrar está activa*/
     bool BorrarActivado() const {return borrar;}
     /** Procedimiento que activa o desactiva la herramienta Relleno Conjunto*/
     void RellenoConjuntoActivar(bool);
     /**  Método que devuelve si la herramienta Relleno Conjunto está activa*/
     bool RellenoConjuntoActivado() {return rellenoConjunto;}
     /** Procedimiento que borra el rectángulo que se le pasa como parametro dentro del mapa.*/
     void Borrar(QRect rect);
     /** Procedimiento que rellena el rectángulo que se le pasa como parametro dentro del mapa con el tile que está seleccionado en la lista de tiles*/
     void Rellenar(QRect rect);
     /** Método recursivo que rellena toda la zona libre sobre el punto que se realiza*/
     int  RellenarRecursiva(QPoint pos);
     /** Procedimiento que rellena toda la zona libre sobre el punto que se realiza, ayudado por la función RellenarRecursiva*/
     void RellenarConjunto(QPoint pos);
     /** Procedimiento que activa o desactiva la herramienta Pegar*/
     void PegarActivar(bool);
     /** Procedimiento que activa o desactiva la rejilla del mapa*/
     void ActivarRejilla(bool);
     /** Procedimiento que realiza el copiado de la zona seleccionada*/
     void Copiar();
     /** Procedimiento que realiza el cortado de la zona seleccionada*/
     void Cortar();
     /** Procedimiento que realiza el pegado de la zona copiada o cortada*/
     void Pegar();
     /** Procedimiento que elimina la zona selecciona*/
     void Eliminar();
     /** Procedimiento que deshace la última acción realizada sobre el mapa*/
     void Deshacer();
     /** Procedimiento que rehace la última acción realizada sobre el mapa*/
     void Rehacer();
     /** Procedimiento que redimensiona el tamaño del mapa al nuevo dado*/
     void Redimensionar(int tanchura, int taltura, int tnumanchura, int tnumaltura);
     /** Procedimiento que inserta el tile y la posición modificada en la lista deshacer*/
     void InsertarRehacer(int tile, QPoint pos);
     /** Procedimiento que inserta el tile y la posición modificada en la lista rehacer*/
     void InsertarDeshacer(int tile, QPoint pos);
     /** Procedimiento que inserta el tipo de acción y el número de tile que conlleva la operación en la lista deshacer*/
     void InsertarNumRehacer(int tipo,int num);
     /** Procedimiento que inserta el tipo de acción y el número de tile que conlleva la operación en la lista rehacer*/
     void InsertarNumDeshacer(int tipo,int num);
     /** Procedimiento que actualiza la lista deshacer cuando se elimina un tile de la lista de tiles*/
     void ActualizarDeshacer(int tile);
     /** Procedimiento que actualiza la lista rehacer cuando se elimina un tile de la lista de tiles*/
     void ActualizarRehacer(int tile);
     /** Método que devuelve el rectángulo del tile correspondiente a un punto del mapa*/
     const QRect TileSeleccionado(const QPoint &posicion);
     /** Método que devuelve si un elemento de la matriz de datos está vacia o no*/
     bool posLibre(int x, int y) const{ return (matrizTiles[x][y])==-1;}
     /** Método que devulve si una posición dada está dentro de los límites del mapa o no*/
     bool posCorrecta(QPoint pos) { return pos.x() < AnchuraTileZoom()*NumAnchuraTile() && pos.x() >= 0 &&
                                    pos.y() < AlturaTileZoom()*NumAlturaTile() && pos.y() >=0; }
 protected:
     /** Procedimiento que se activa cada vez que se produce un evento de entrada de dibujo sobre la superficie del mapa*/
     void dragEnterEvent(QDragEnterEvent *event);
     /** Procedimiento que se activa cada vez que se produce un evento de salida de dibujo sobre la superficie del mapa*/
     void dragLeaveEvent(QDragLeaveEvent *event);
     /** Procedimiento que se activa cada vez que se produce un evento de movimiento de dibujo sobre la superficie del mapa*/
     void dragMoveEvent(QDragMoveEvent *event);
     /** Procedimiento que se activa cada vez que se produce un evento de soltado sobre la superficie del mapa*/
     void dropEvent(QDropEvent *event);
     /** Procedimiento que se activa cada vez que se produce un evento de presión con el ratón sobre la superficie del mapa*/
     void mousePressEvent(QMouseEvent *event);
     /** Procedimiento que se activa cada vez que se produce un evento de movimiento con el ratón sobre la superficie del mapa*/
     void mouseMoveEvent(QMouseEvent *);
     /** Procedimiento que se activa cada vez que se produce una actualización sobre la superficie del mapa*/
     void paintEvent(QPaintEvent *event);
     /** Procedimiento que se activa cada vez que el puntero del ratón sale de los límites del mapa*/
     void leaveEvent(QEvent *event);

 private:
     int **matrizTiles;

     QList<int> tilescopiar,tilesdeshacer,tilesrehacer,tilemov,tilemovr;
     QList<QPoint> poscopiar,posdeshacer,posrehacer,posmovant,posmovact,posmovantr,posmovactr;
     QList<QPoint> numdeshacer,numrehacer;
     QRect rectcopiar;
     QRect rectsel,rectselant,rectselsi,rectpress;

     QLineEdit prueba;
     QLineEdit prueba1;

     QColor crejilla,cfondo,csel;
     Tile *listatiles;
     Minimapa *minimapa;
     VentanaPrincipal * ventana;

     bool salida,mov,seleccion,relleno,borrar,rellenoConjunto,rejillaactiva,rejillapintada,pegado;
     int modificado,contador,contador1,pintarc,pintarcompleto,factorzoom;
     int tanchura,taltura,tnumanchura,tnumaltura,tanchuraZ,talturaZ;
 };

 #endif
