#include "etilessdl.h"
#include <fstream>
#include <iterator>
#include <math.h>

using namespace std;

/* coherent noise function over 1, 2 or 3 dimensions */
/* (copyright Ken Perlin) */
/*This source provides a C++ wrapper around Ken Perlin's noise function. I know there already is a Perlin noise function on the COTD collection, but this one serves a specific purpose. The C++ wrapper allows you to create multiple instances of perlin noise functions so you can have completely determinstic and distinct noise textures. Each instance of the 'Perlin' class can be sampled independently of the other, always returning the same randomized results. You construct an instance of Perlin as follows: Perlin *perlin = new Perlin(4,4,1,94); The first parameter is the number of octaves, this is how noisy or smooth the function is. This is valid between 1 and 16. A value of 4 to 8 octaves produces fairly conventional noise results. The second parameter is the noise frequency. Values betwen 1 and 8 are reasonable here. You can try sampling the data and plotting it to the screen to see what numbers you like. The last parameter is the amplitude. Setting this to a value of 1 will return randomized samples between -1 and +1. The last parameter is the random number seed. This number is what causes this instance of the Perlin to be deterministic and distanct from any other instance. The perlin noise function creates some random number tables that are sampled during queries. This random number seed determines the contents of those tables so that you will get the same exact results every time you use it. To retrieve a sample you simply call the method 'Get' and pass it the X and Y sample point to query. X and Y should be in the ranges of 0 to 1. So if you are sampling for a bitmap be sure and scale the pixel co-ordinates down into normalized values. Each instance of Perlin contains it's own random number tables and sampling values. This class is extremely convenient if you just need a quick and dirty way to do some kind of distribution pattern that looks halfway decent.*/

#define B SAMPLE_SIZE
#define BM (SAMPLE_SIZE-1)

#define N 0x1000
#define NP 12   /* 2^N */
#define NM 0xfff

#define s_curve(t) ( t * t * (3.0f - 2.0f * t) )
#define lerp(t, a, b) ( a + t * (b - a) )

#define setup(i,b0,b1,r0,r1)\
	t = vec[i] + N;\
	b0 = ((int)t) & BM;\
	b1 = (b0+1) & BM;\
	r0 = t - (int)t;\
	r1 = r0 - 1.0f;


float Perlin::noise1(float arg)
{
	int bx0, bx1;
	float rx0, rx1, sx, t, u, v, vec[1];

	vec[0] = arg;

	if (mStart)
  {
    srand(mSeed);
		mStart = false;
		init();
	}

	setup(0, bx0,bx1, rx0,rx1);

	sx = s_curve(rx0);

	u = rx0 * g1[ p[ bx0 ] ];
	v = rx1 * g1[ p[ bx1 ] ];

	return lerp(sx, u, v);
}

float Perlin::noise2(float vec[2])
{
	int bx0, bx1, by0, by1, b00, b10, b01, b11;
	float rx0, rx1, ry0, ry1, *q, sx, sy, a, b, t, u, v;
	int i, j;

	if (mStart)
  {
    srand(mSeed);
		mStart = false;
		init();
	}

	setup(0,bx0,bx1,rx0,rx1);
	setup(1,by0,by1,ry0,ry1);

	i = p[bx0];
	j = p[bx1];

	b00 = p[i + by0];
	b10 = p[j + by0];
	b01 = p[i + by1];
	b11 = p[j + by1];

	sx = s_curve(rx0);
	sy = s_curve(ry0);

  #define at2(rx,ry) ( rx * q[0] + ry * q[1] )

	q = g2[b00];
	u = at2(rx0,ry0);
	q = g2[b10];
	v = at2(rx1,ry0);
	a = lerp(sx, u, v);

	q = g2[b01];
	u = at2(rx0,ry1);
	q = g2[b11];
	v = at2(rx1,ry1);
	b = lerp(sx, u, v);

	return lerp(sy, a, b);
}

float Perlin::noise3(float vec[3])
{
	int bx0, bx1, by0, by1, bz0, bz1, b00, b10, b01, b11;
	float rx0, rx1, ry0, ry1, rz0, rz1, *q, sy, sz, a, b, c, d, t, u, v;
	int i, j;

	if (mStart)
  {
    srand(mSeed);
		mStart = false;
		init();
	}

	setup(0, bx0,bx1, rx0,rx1);
	setup(1, by0,by1, ry0,ry1);
	setup(2, bz0,bz1, rz0,rz1);

	i = p[ bx0 ];
	j = p[ bx1 ];

	b00 = p[ i + by0 ];
	b10 = p[ j + by0 ];
	b01 = p[ i + by1 ];
	b11 = p[ j + by1 ];

	t  = s_curve(rx0);
	sy = s_curve(ry0);
	sz = s_curve(rz0);

  #define at3(rx,ry,rz) ( rx * q[0] + ry * q[1] + rz * q[2] )

	q = g3[ b00 + bz0 ] ; u = at3(rx0,ry0,rz0);
	q = g3[ b10 + bz0 ] ; v = at3(rx1,ry0,rz0);
	a = lerp(t, u, v);

	q = g3[ b01 + bz0 ] ; u = at3(rx0,ry1,rz0);
	q = g3[ b11 + bz0 ] ; v = at3(rx1,ry1,rz0);
	b = lerp(t, u, v);

	c = lerp(sy, a, b);

	q = g3[ b00 + bz1 ] ; u = at3(rx0,ry0,rz1);
	q = g3[ b10 + bz1 ] ; v = at3(rx1,ry0,rz1);
	a = lerp(t, u, v);

	q = g3[ b01 + bz1 ] ; u = at3(rx0,ry1,rz1);
	q = g3[ b11 + bz1 ] ; v = at3(rx1,ry1,rz1);
	b = lerp(t, u, v);

	d = lerp(sy, a, b);

	return lerp(sz, c, d);
}

void Perlin::normalize2(float v[2])
{
	float s;

	s = (float)sqrt(v[0] * v[0] + v[1] * v[1]);
  s = 1.0f/s;
	v[0] = v[0] * s;
	v[1] = v[1] * s;
}

void Perlin::normalize3(float v[3])
{
	float s;

	s = (float)sqrt(v[0] * v[0] + v[1] * v[1] + v[2] * v[2]);
  s = 1.0f/s;

	v[0] = v[0] * s;
	v[1] = v[1] * s;
	v[2] = v[2] * s;
}

void Perlin::init(void)
{
	int i, j, k;

	for (i = 0 ; i < B ; i++)
  {
		p[i] = i;
		g1[i] = (float)((rand() % (B + B)) - B) / B;
		for (j = 0 ; j < 2 ; j++)
			g2[i][j] = (float)((rand() % (B + B)) - B) / B;
		normalize2(g2[i]);
		for (j = 0 ; j < 3 ; j++)
			g3[i][j] = (float)((rand() % (B + B)) - B) / B;
		normalize3(g3[i]);
	}

	while (--i)
  {
		k = p[i];
		p[i] = p[j = rand() % B];
		p[j] = k;
	}

	for (i = 0 ; i < B + 2 ; i++)
  {
		p[B + i] = p[i];
		g1[B + i] = g1[i];
		for (j = 0 ; j < 2 ; j++)
			g2[B + i][j] = g2[i][j];
		for (j = 0 ; j < 3 ; j++)
			g3[B + i][j] = g3[i][j];
	}

}


float Perlin::perlin_noise_2D(float vec[2])
{
  int terms    = mOctaves;
	float freq   = mFrequency;
	float result = 0.0f;
  float amp = mAmplitude;

  vec[0]*=mFrequency;
  vec[1]*=mFrequency;

	for( int i=0; i<terms; i++ )
	{
		result += noise2(vec)*amp;
		vec[0] *= 2.0f;
		vec[1] *= 2.0f;
    amp*=0.5f;
	}


	return result;
}



Perlin::Perlin(int octaves,float freq,float amp,int seed)
{
  mOctaves = octaves;
  mFrequency = freq;
  mAmplitude = amp;
  mSeed = seed;
  mStart = true;
}

/******************************************************************************************************************/
/********************************************CLASE SDLETiles*******************************************************/
/******************************************************************************************************************/

SDLETiles::SDLETiles()
{
	tanchura=0;
	taltura=0;
	tnumalt=0;
	tnumanc=0;
}

/*SDLETiles::SDLETiles(int anchura,int altura, int numanchura, int numaltura) : tanchura(anchura), taltura(altura), tnumanc(numanchura), tnumalt(numaltura)
{
     matrizTiles = new int*[tnumanc];

     for(int i=0;i<tnumanc;i++)
     {
         matrizTiles[i] = new int[tnumalt];
     }
     for(int i=0;i<tnumanc;i++)
         for(int j=0;j<tnumalt;j++)
             matrizTiles[i][j]=-1; //Valor por defecto
}*/

void SDLETiles::aleatorioPerlin(int octaves, double frecuencia, double amplitud, int semilla,double margen)
{
    Perlin *p;
    p = new Perlin(octaves,frecuencia,amplitud,semilla);
    int ntile,tsel;
    double valor=0;
    for(int i=0;i<NumAnchuraTile();i++)
    {
        for(int j=0;j<NumAlturaTile();j++)
        {
               valor = p->Get(double(i)/double(NumAnchuraTile()),double(j)/double(NumAlturaTile()));
               ntile = 0;
               bool enc=false;
               for(double k=-1+margen; k<=1 && !enc; k += margen)
               {
                   if(valor < k)
                   {
                        tsel=ntile;
                        enc=true;
                   }
                   ntile++;
               }
               setTilePosicion(tsel,i,j);
        }
    }
}

void SDLETiles::crearMapaAleatorio(int tanchura,int taltura, int tnumanchura, int tnumaltura,list<SDL_Surface*> ltiles, int aguatierra, int contislas, int monvar, int semilla,int eleccion)
{
     this->tanchura=tanchura;
     this->taltura=taltura;
     tnumanc=tnumanchura;
     tnumalt=tnumaltura;
     matrizTiles = new int*[tnumanchura];

     for(int i=0;i<tnumanchura;i++)
     {
         matrizTiles[i] = new int[tnumaltura];
     }
     for(int i=0;i<tnumanchura;i++)
         for(int j=0;j<tnumaltura;j++)
             matrizTiles[i][j]=-1;

    listatiles=ltiles;
    //listarutastiles=lrutas;
    if(listatiles.size() > 0)
    {
/*        for(int i=0;i<ltiles.size();i++)
        {
            listatiles.append(ltiles.at(i));
	    listarutatiles.append(lrutas.at(i));
        }*/

        if(eleccion==0)
        {
            double margen=2./ltiles.size();
            int octaves=(int)(aguatierra/20)+4;
            double frecuencia=(contislas/13.)+1; 
            double amplitud=(monvar*2-100)/100.;
            aleatorioPerlin(octaves,frecuencia,amplitud,semilla,margen);
        }
        //else if(eleccion==1)
        //{

        //}
    }
}

void SDLETiles::cargarMapa(const char * ruta, const char *rutatiles)
{
    ifstream fs(ruta);

    int anchura,altura,numanchura,numaltura,numtile;
    string nombre,rutafinal;
    list<string> lrutas;
    list<int> lseparacion;
    list<SDL_Surface> ltiles;
    SDL_Surface itile;
    char caraux;

    fs >> anchura; tanchura=anchura;//setAnchuraTile(anchura);
    fs >> altura; taltura=altura;//setAlturaTile(altura);
    fs >> numanchura; tnumanc=numanchura;//setNumAnchuraTile(numanchura);
    fs >> numaltura; tnumalt=numaltura;//setNumAlturaTile(numaltura);

     matrizTiles = new int*[tnumanc];

     for(int i=0;i<tnumanc;i++)
     {
         matrizTiles[i] = new int[tnumalt];
     }
     for(int i=0;i<tnumanc;i++)
         for(int j=0;j<tnumalt;j++)
             matrizTiles[i][j]=-1;

    for(int j=0; j < NumAlturaTile(); j++)
    {
         for(int i=0; i < NumAnchuraTile(); i++)
         {
	      fs >> numtile; fs >> caraux;
              setTilePosicion(numtile,i,j);
         }
     }
     while(fs >> numtile)
     {
	 fs >> caraux;
	 fs >> nombre;
	 lseparacion.push_back(numtile);
	 lrutas.push_back(nombre);		 
     }
     fs.close();

     list<int>::iterator itsep = lseparacion.begin();
     for(list<string>::iterator i = lrutas.begin(); i!=lrutas.end(); i++)
     {
	 int tileset = 1;
	 list<string>::iterator k=i; k++;

	 while( k != lrutas.end() &&  *i == *k )
         {
		tileset++;
		k++;
         }
	 if(tileset == 1) //Es un tile unico
         {
		rutafinal = rutatiles + *i;
		SDL_Surface * itile; if(!(itile = IMG_Load(rutafinal.c_str()))) exit(1);
		listatiles.push_back(itile);
 	 }
	 else //Es un tileset
	 {
		int margen = *itsep;			
		SDL_Surface *tile;
		rutafinal = rutatiles + *i;
		SDL_Surface * itileset = IMG_Load(rutafinal.c_str());
		SDL_Rect rect; rect.x=0+margen; rect.y=0+margen; rect.w=AnchuraTile()-margen; rect.h=AlturaTile()-margen;
		for(int cont=0; cont<tileset; cont++){
			/* SDL interprets each pixel as a 32-bit number, so our masks must depend
			       on the endianness (byte order) of the machine 
			Uint32 rmask, gmask, bmask, amask;
			#if SDL_BYTEORDER == SDL_BIG_ENDIAN
			    rmask = 0xff000000;
			    gmask = 0x00ff0000;
			    bmask = 0x0000ff00;
			    amask = 0x000000ff;
			#else
			    rmask = 0x000000ff;
			    gmask = 0x0000ff00;
			    bmask = 0x00ff0000;
			    amask = 0xff000000;
			#endif*/
			 tile = SDL_CreateRGBSurface(SDL_SWSURFACE,
					     AnchuraTile(),
					     AlturaTile(), 24, 0, 0, 0, 0);

			
			SDL_BlitSurface(itileset,&rect,tile,NULL);
			rect.y += AlturaTile()+margen;
			if(rect.y >= itileset->h)
			{
				rect.y =0+margen;
				rect.x += AnchuraTile()+margen;
			}

			listatiles.push_back(tile);			
		}
		i=k;
		i--;
 	 }
     }

}

void SDLETiles::pintarEnSuperficie(SDL_Surface *superficie)
{

    int t;
    SDL_Surface *itile;
    SDL_Rect rect; rect.x=0; rect.y=0; rect.w=AnchuraTile(); rect.h=AlturaTile();
    for(int i=0; i < NumAnchuraTile(); i++)
    {
         for(int j=0; j < NumAlturaTile(); j++)
         {
		t = TilePosicion(i,j);
		if(t!=-1) //No esta vacio
		{
			int k=0;
			bool enc=false;
			for(list<SDL_Surface*>::iterator it=listatiles.begin(); it!= listatiles.end() && !enc ; it++)
			{
				if(t==k)
				{
					SDL_BlitSurface(*it,NULL,superficie,&rect);
					//itile = *it;
					enc=true;
				}
				k++;
			}
		}
		rect.y += AlturaTile();
	 }
	 rect.x += AnchuraTile();
	 rect.y = 0;
    }
}
